# Forcibly re-enable assertions, even if we're building in release
# mode. This is a security project - assertions may be enforcing
# security-critical constraints. A backstop #ifdef in defs.h should
# give a #error if this manoeuvre doesn't do what it needs to.
string(REPLACE "/DNDEBUG" "" CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE}")
string(REPLACE "-DNDEBUG" "" CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE}")

set(XTRUSS_DEBUG OFF
  CACHE BOOL "Build xtruss with debug() statements enabled")
set(XTRUSS_COVERAGE OFF
  CACHE BOOL "Build xtruss binaries suitable for code coverage analysis")

set(STRICT OFF
  CACHE BOOL "Enable extra compiler warnings and make them errors")

include(FindGit)

set(GENERATED_SOURCES_DIR ${CMAKE_CURRENT_BINARY_DIR}${CMAKE_FILES_DIRECTORY})

set(GENERATED_LICENCE_H ${GENERATED_SOURCES_DIR}/licence.h)
set(INTERMEDIATE_LICENCE_H ${GENERATED_LICENCE_H}.tmp)
add_custom_command(OUTPUT ${INTERMEDIATE_LICENCE_H}
  COMMAND ${CMAKE_COMMAND}
    -DLICENCE_FILE=${CMAKE_SOURCE_DIR}/LICENCE
    -DOUTPUT_FILE=${INTERMEDIATE_LICENCE_H}
    -P ${CMAKE_SOURCE_DIR}/cmake/licence.cmake
  DEPENDS ${CMAKE_SOURCE_DIR}/cmake/licence.cmake ${CMAKE_SOURCE_DIR}/LICENCE)
add_custom_target(generated_licence_h
  BYPRODUCTS ${GENERATED_LICENCE_H}
  COMMAND ${CMAKE_COMMAND} -E copy_if_different
    ${INTERMEDIATE_LICENCE_H} ${GENERATED_LICENCE_H}
  DEPENDS ${INTERMEDIATE_LICENCE_H}
  COMMENT "Updating licence.h")

set(GENERATED_COMMIT_C ${GENERATED_SOURCES_DIR}/cmake_commit.c)
set(INTERMEDIATE_COMMIT_C ${GENERATED_COMMIT_C}.tmp)
add_custom_target(check_git_commit
  BYPRODUCTS ${INTERMEDIATE_COMMIT_C}
  COMMAND ${CMAKE_COMMAND}
    -DGIT_EXECUTABLE=${GIT_EXECUTABLE}
    -DOUTPUT_FILE=${INTERMEDIATE_COMMIT_C}
    -DOUTPUT_TYPE=header
    -P ${CMAKE_SOURCE_DIR}/cmake/gitcommit.cmake
  DEPENDS ${CMAKE_SOURCE_DIR}/cmake/gitcommit.cmake
  WORKING_DIRECTORY ${CMAKE_SOURCE_DIR}
  COMMENT "Checking current git commit")
add_custom_target(cmake_commit_c
  BYPRODUCTS ${GENERATED_COMMIT_C}
  COMMAND ${CMAKE_COMMAND} -E copy_if_different
    ${INTERMEDIATE_COMMIT_C} ${GENERATED_COMMIT_C}
  DEPENDS check_git_commit ${INTERMEDIATE_COMMIT_C}
  COMMENT "Updating cmake_commit.c")

function(add_sources_from_current_dir target)
  set(sources ${ARGN})
  list(TRANSFORM sources PREPEND ${CMAKE_CURRENT_SOURCE_DIR}/)
  target_sources(${target} PRIVATE ${sources})
endfunction()

set(extra_dirs)
# Here we could insert additional platform support if needed
set(platform unix)

find_package(Python3 COMPONENTS Interpreter)
find_package(PkgConfig)
set(XTRUSS_USING_XTRGEN FALSE)
if(PKG_CONFIG_FOUND)
  pkg_check_modules(XCBPROTO xcb-proto)
  if(XCBPROTO_FOUND AND Python3_FOUND)
    set(XTRUSS_USING_XTRGEN TRUE)
    add_compile_definitions(USING_XTRGEN)
    pkg_get_variable(XCBPROTODIR xcb-proto xcbincludedir)
    set(XTRGEN_SOURCES # Comments explain why headers are not used.
      # bigreq is supported natively
      composite
      damage
      # dbe needs nested <list>
      dpms
      dri2
      # dri3 needs <fd>
      # ge is supported natively
      # glx needs type="double"
      # present needs <field type="CARD64">
      # randr needs nested <list> and four resource types
      # record needs nested <list> and CONTEXT resources
      # render is supported natively
      # res needs nested <list>
      screensaver
      shape
      # shm is supported natively
      sync
      xc_misc
      xevie
      xf86dri
      xf86vidmode
      xfixes
      xinerama
      # xinput needs nested <list>
      # xkb needs <popcount> and <sumof>
      # xprint needs multiple <valueparam>s in a message
      # xproto is supported natively
      xselinux
      xtest
      # xvmc needs <fd>
      # xv needs <fd>
      )
    list(TRANSFORM XTRGEN_SOURCES PREPEND ${XCBPROTODIR}/)
    list(TRANSFORM XTRGEN_SOURCES APPEND .xml)
    add_custom_command(
      OUTPUT ${CMAKE_BINARY_DIR}/xtruss-generated.c
      COMMAND Python3::Interpreter ${CMAKE_SOURCE_DIR}/xtrgen.py
        ${XTRGEN_SOURCES}        
        > ${CMAKE_BINARY_DIR}/xtruss-generated.c
      DEPENDS xtrgen.py
      VERBATIM)
    add_custom_target(run_xtrgen
      DEPENDS ${CMAKE_BINARY_DIR}/xtruss-generated.c)
    set_property(
      SOURCE ${CMAKE_BINARY_DIR}/xtruss-generated.c
      PROPERTY HEADER_FILE_ONLY ON)
  endif()
endif()

include(cmake/platforms/${platform}.cmake)

include_directories(
  ${CMAKE_CURRENT_SOURCE_DIR}
  ${GENERATED_SOURCES_DIR}
  ${platform}
  ${extra_dirs})

if(XTRUSS_DEBUG)
  add_compile_definitions(DEBUG)
endif()
if(XTRUSS_COVERAGE)
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -fprofile-arcs -ftest-coverage -g ")
endif()
